package org.xbib.interlibrary.http.server.avail;

import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.xbib.interlibrary.http.server.BaseView;
import org.xbib.j2html.tags.DomContent;
import org.xbib.j2html.tags.Tag;
import org.xbib.j2html.tags.specialized.MainTag;
import org.xbib.net.ParameterException;
import org.xbib.net.http.server.HttpRequest;
import org.xbib.net.http.server.application.Application;
import org.xbib.net.http.server.resource.HtmlTemplateResource;
import org.xbib.net.http.server.route.HttpRouterContext;
import org.xbib.net.http.server.session.Session;
import static org.xbib.j2html.TagCreator.div;
import static org.xbib.j2html.TagCreator.each;
import static org.xbib.j2html.TagCreator.form;
import static org.xbib.j2html.TagCreator.h1;
import static org.xbib.j2html.TagCreator.input;
import static org.xbib.j2html.TagCreator.main;
import static org.xbib.j2html.TagCreator.option;
import static org.xbib.j2html.TagCreator.select;
import static org.xbib.j2html.TagCreator.span;

public class AvailFormView extends BaseView {

    private static final Logger logger = Logger.getLogger(AvailFormView.class.getName());

    private String formId;

    public AvailFormView(HtmlTemplateResource resource,
                         Application application,
                         HttpRequest request) throws ParameterException {
        super(resource, application, request);
    }

    @Override
    protected DomContent mainContent(HttpRouterContext context, Tag<?>... tags) {
        AvailApplicationModule app = context.getAttributes().get(AvailApplicationModule.class, "app");
        Objects.requireNonNull(app, "app must not be null in context");
        Session session = context.getAttributes().get(Session.class, "session");
        Objects.requireNonNull(session, "session must not be null in context");
        Map<String, Object> availform = app.getAvailForm("availform", request, session);
        Map<String, Object> availresponse = app.getAvailResponse("_availresponse", session);
        this.formId = (String) availform.get("_id");
        AvailResultView availResultView = new AvailResultView(availform);
        MainTag mainTag = main(availForm(availform), availResultView.availResult(availresponse));
        app.removeAvailResponse("_availresponse", session);
        return mainTag;

    }

    @SuppressWarnings("unchecked")
    private DomContent availForm(Map<String, Object> availForm) {

        List<AvailForm.Key> sources = (List<AvailForm.Key>) availForm.get("sources");
        List<AvailForm.Key> modes = (List<AvailForm.Key>) availForm.get("modes");
        List<AvailForm.Key> materials = (List<AvailForm.Key>) availForm.get("materials");
        List<AvailForm.Key> domains = (List<AvailForm.Key>) availForm.get("domains");

        return div().attr("class", "container").with(
                h1("Verfügbarkeitsrecherche"),
                form().attr("class", "form-horizontal mt-1 mb-1")
                        .attr("id", "availform")
                        .attr("method", "post")
                        .attr("action", resource.contextPath(request, "action/availform/" + formId)).with(
                                div().attr("class", "row mb-1 input-group").with(
                                        span().attr("class", "col-sm-2").with(
                                                select().attr("class", "form-select border border-secondary")
                                                .attr("id", "source")
                                                .attr("name", "source")
                                                .with(each(sources, entry ->
                                                        option().attr("value", entry.getValue())
                                                                .attr(entry.isDisabled() ? "disabled" : entry.isSelected() ? "selected" : "")
                                                                .withText(entry.getLabel())
                                                ))
                                        ),
                                        span().attr("class", "col-sm-2").with(
                                                input().attr("class", "form-control")
                                                        .attr("type", "text")
                                                        .attr("name", "sourceid")
                                                        .attr("value", availForm.get("sourceid"))
                                                        .attr("placeholder", "Quellen-ID")
                                        ),
                                        span().attr("class", "col-sm-2").with(
                                                input().attr("class", "form-control")
                                                        .attr("type", "number")
                                                        .attr("name", "year")
                                                        .attr("value", availForm.get("year"))
                                                        .attr("placeholder", "Jahr")
                                        ),
                                        span().attr("class", "col-sm-2").with(
                                                select().attr("class", "form-select")
                                                        .attr("id", "mode")
                                                        .attr("name", "mode")
                                                        .with(each(modes, entry ->
                                                                option().attr("value", entry.getValue())
                                                                        .attr(entry.isDisabled() ? "disabled" : entry.isSelected() ? "selected" : "")
                                                                        .withText(entry.getLabel())
                                                        ))
                                        ),
                                        span().attr("class", "col-sm-2").with(
                                                select().attr("class", "form-select")
                                                        .attr("id", "material")
                                                        .attr("name", "material")
                                                        .with(each(materials, entry ->
                                                                option().attr("value", entry.getValue())
                                                                        .attr(entry.isDisabled() ? "disabled" : entry.isSelected() ? "selected" : "")
                                                                        .withText(entry.getLabel())
                                                        ))
                                        ),
                                        span().attr("class", "col-sm-2").with(
                                                select().attr("class", "form-select")
                                                        .attr("id", "domain")
                                                        .attr("name", "domain")
                                                        .with(each(domains, entry ->
                                                                option().attr("value", entry.getValue())
                                                                        .attr(entry.isDisabled() ? "disabled" : entry.isSelected() ? "selected" : "")
                                                                        .withText(entry.getLabel())
                                                        ))
                                        )
                                ),
                                div().attr("class", "d-grid gap-2 d-md-flex justify-content-md-end").with(
                                        input().attr("class", "btn btn-outline-primary")
                                                .attr("type", "submit")
                                                .attr("id", "availbutton")
                                                .attr("name", "availbutton")
                                                .attr("value", "Verfügbarkeit ermitteln >>")
                                                .attr("onclick", "javascript:pleaseWait('availbutton', 'availform')")
                                )
                        )
                );
    }
}
