package org.xbib.interlibrary.http.server.avail;

import org.xbib.interlibrary.api.util.Maps;
import org.xbib.j2html.tags.DomContent;
import org.xbib.j2html.tags.DomContentJoiner;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import static org.xbib.j2html.TagCreator.a;
import static org.xbib.j2html.TagCreator.button;
import static org.xbib.j2html.TagCreator.div;
import static org.xbib.j2html.TagCreator.each;
import static org.xbib.j2html.TagCreator.h4;
import static org.xbib.j2html.TagCreator.i;
import static org.xbib.j2html.TagCreator.p;
import static org.xbib.j2html.TagCreator.rawHtml;
import static org.xbib.j2html.TagCreator.span;
import static org.xbib.j2html.TagCreator.table;
import static org.xbib.j2html.TagCreator.tbody;
import static org.xbib.j2html.TagCreator.td;
import static org.xbib.j2html.TagCreator.th;
import static org.xbib.j2html.TagCreator.thead;
import static org.xbib.j2html.TagCreator.tr;

public class AvailResultView {

    private final List<AvailForm.Key> modes;

    private final List<AvailForm.Key> materials;

    @SuppressWarnings("unchecked")
    public AvailResultView(Map<String, Object> availForm) {
        modes = (List<AvailForm.Key>) availForm.get("modes");
        materials = (List<AvailForm.Key>) availForm.get("materials");
    }

    @SuppressWarnings("unchecked")
    public DomContent availResult(Map<String, Object> availResult) {
        if (availResult == null) {
            return div().attr("class", "container").with(
                    div().attr("class", "row").with(
                            div().attr("class", "col-md-12").with(
                                    p("Es konnten keine Ergebnisse gefunden werden.")
                            )
                    )
            );
        }
        Map<String, Object> meta = (Map<String, Object>) availResult.get("meta");
        return div().attr("class", "container").with(
                div().attr("class", "row").with(
                        div().attr("class", "col-md-12").with(
                                links(availResult)
                        )
                ),
                div().attr("class", "row").with(
                        div().attr("class", "col-md-12").with(
                                displayAvail(meta, (Map<String, Object>) availResult.get("selflibrary"),
                                        "Eigene Bestände und Lizenzen", "selflibrary"),
                                displayAvail(meta, (Map<String, Object>) availResult.get("locallibrary"),
                                        "Bestände und Lizenzen am Ort", "locallibrary"),
                                displayAvail(meta, (Map<String, Object>) availResult.get("interlibrary"),
                                        "Bestände und Lizenzen", "interlibrary"),
                                displayAvail(meta, (Map<String, Object>) availResult.get("interlibrarydomains"),
                                        "Bestände und Lizenzen für die überregionale Fernleihe", "interlibrarydomains"),
                                displayAvail(meta, (Map<String, Object>) availResult.get("lastresortlibrary"),
                                        "Bestände und Lizenzen der Nationalbibliotheken", "lastresortlibrary"),
                                displayAvail(meta, (Map<String, Object>) availResult.get("filteredlibrary"),
                                        "Nicht berücksichtigte Bestände und Lizenzen", "filteredlibrary")
                        )
                )
        );
    }

    private DomContent links(Map<String, Object> availResult) {
        List<Map<String, Object>> links = Maps.getListMap(availResult, "meta.link");
        if (links == null || links.isEmpty()) {
            return null;
        }
        return div().attr("class", "card").with(
                div().attr("class", "card-header").with(
                        div("Dieser Titel verfügt über Online-Ressourcen")
                ),
                div().attr("class", "card-body").with(
                        openAccess(availResult),
                        each(links, link ->
                                div().with(
                                        i().attr("class", "bi-link").with(rawHtml("&nbsp;")),
                                        a().attr("href", link.get("url")).attr("_target", "blank")
                                                .withText(link.containsKey("label") ? (String) link.get("label") :""),
                                        span(link.containsKey("label") && link.get("label").toString().contains("kostenfrei") ?
                                                " Diese Ressource ist grün" : "")
                                )
                        )
                )
        );
    }

    private DomContent openAccess(Map<String, Object> availResult) {
        if (Maps.getBoolean(availResult, "meta.openaccess")) {
            return DomContentJoiner.join("", true, rawHtml("""
<svg viewbox="0 0 640 1000" width="16" height="25">
    <rect width="640" height="1000" fill="none"/>
    <g stroke="#f68212" stroke-width="104.764" fill="none">
        <path d="M111.387,308.135V272.408A209.21,209.214 0 0,1 529.807,272.408V530.834"/>
        <circle cx="320.004" cy="680.729" r="256.083"/>
    </g>
    <circle fill="#f68212" cx="321.01" cy="681.659" r="86.4287"/>
</svg>&nbsp;
"""),
                    span("Das ist eine Open-Access-Ressource, die für alle frei verfügbar ist."));
        } else {
            return null;
        }
    }

    private DomContent displayAvail(Map<String, Object> meta,
                                    Map<String, Object> map,
                                    String title,
                                    String id) {
        if (map == null || map.isEmpty()) {
            return null;
        }
        return DomContentJoiner.join("", true,
                h4().attr("class", "mt-3").withText(title),
                div().attr("class", "accordion").attr("id", id).with(
                        each(map.entrySet(), l -> libraryAvailEntry(meta, l, id))
                )
        );
    }

    @SuppressWarnings("unchecked")
    private DomContent libraryAvailEntry(Map<String, Object> meta,
                                         Map.Entry<String, Object> entry,
                                         String id) {
        String isil = entry.getKey();
        String target = "collapse_" + id + "_" + isil;
        List<Map<String, Object>> services = (List<Map<String, Object>>) entry.getValue();
        Map<String, Object> firstService = services.isEmpty() ? Map.of() : services.getFirst();
        Long priority = (Long) firstService.get("priority");
        priority = priority != null ? priority : 9;
        String name = (String) ((Map<String, Object>) meta.get("isil")).get(isil);
        if (name == null) {
            name = (String) firstService.get("name");
        }
        return div().attr("class", "accordion-item").with(
                h4().attr("class", "accordion-header")
                        .attr("id", "heading_" + target).with(
                        button().attr("class", "accordion-button")
                                .attr("style", "color: black; background-color: white")
                                .attr("type", "button")
                                .attr("data-bs-toggle", "collapse")
                                .attr("data-bs-target", "#" + target)
                                .attr("aria-expanded", "true")
                                .attr("aria-controls", target).with(
                                        span().attr("class", colorClass(priority)).withText(priority.toString()),
                                        span().attr("class", "px-1").withText(isil),
                                        span().attr("class", "px-1").withText(name)
                                )
                ),
                div().attr("id", target)
                        .attr("class", "accordion-collapse collapse")
                        .attr("aria-labelledby", "heading_" + target)
                        .attr("data-bs-parent", "#accordion_" + id).with(
                                div().attr("class", "accordion-body").with(
                                        table().attr("class", "table").with(
                                            thead().with(
                                                tr().with(
                                                    th().withText("Bibliothek"),
                                                    th().withText("Standort / Signatur / Lizenz"),
                                                    th().withText("nachgewiesen"),
                                                    th().withText("verfügbar als"),
                                                    th().withText("Fernleihe"),
                                                    th().withText("Priorität")
                                                )
                                            ),
                                            tbody().with(
                                                    each(services, this::serviceRow)
                                            )
                                        )
                                )
                        )
        );
    }

    private DomContent serviceRow(Map<String, Object> service) {
        Long priority = (Long) service.get("priority");
        priority = priority != null ? priority : 9;
        String isil = (String) service.get("isil");
        String name = (String) service.get("name");
        return tr().with(
                td().with(
                        span().attr("class", colorClass(priority)).withText(priority.toString()),
                        span().attr("class", "px-1").withText(isil),
                        span().attr("class", "px-1").withText(name)
                ),
                td().with(rawHtml(formatLocation(service))),
                td().with(rawHtml(formatTextualHoldings(service))),
                td().withText(formatMaterial(service)),
                td().withText(formatMode(service)),
                td().withText(formatPriority(service))
        );
    }

    private String colorClass(long priority) {
        return switch ((int) priority) {
            case 1 -> "badge bg-success mr-1";
            case 2 -> "badge bg-info mr-1";
            case 3 -> "badge bg-warning mr-1";
            case 4 -> "badge bg-primary mr-1";
            case 5 -> "badge bg-secondary mr-1";
            case 9 -> "badge bg-danger mr-1";
            default -> "badge mr-1";
        };
    }

    @SuppressWarnings("unchecked")
    private String formatLocation(Map<String, Object> service) {
        List<String> list = new ArrayList<>();
        if (service.containsKey("location")) {
            List<Map<String, Object>> l = (List<Map<String, Object>>) service.get("location");
            for (Map<String, Object> m : l) {
                list.add(formatSingleLocation(m));
            }
        }
        return String.join(", ", list.stream().filter(Objects::nonNull).filter(s -> !s.isEmpty()).toList());
    }


    @SuppressWarnings("unchecked")
    private static String formatSingleLocation(Map<String, Object> map) {
        List<String> list = new ArrayList<>();
        addString(map, "library", list);
        addString(map, "libraryname", list);
        addString(map, "collection", list);
        addString(map, "callnumber", list);
        addString(map, "description", list);
        if (map.containsKey("links")) {
            Object object = map.get("links");
            if (object instanceof List) {
                list.add(formatLinks((List<Map<String, Object>>) object));
            }
        }
        return String.join(", ", list.stream().filter(Objects::nonNull).filter(s -> !s.isEmpty()).toList());
    }

    private static String formatLinks(List<Map<String, Object>> list) {
        List<String> result = new ArrayList<>();
        for (Map<String, Object> m : list) {
            formatSingleLink(result, m);
        }
        return String.join("<br/>", result);
    }

    private static void formatSingleLink(List<String> list, Map<String, Object> link) {
        String url = (String) link.get("url");
        String label = (String) link.get("label");
        if (label == null) {
            label = "Link";
        }
        list.add("<i class='bi-link'></i>&nbsp;<a href='" + url + "' target='_blank'>" + label + "</a>");
    }

    private static void addString(Map<String, Object> map, String key, List<String> list) {
        if (map.containsKey(key)) {
            Object object = map.get(key);
            if (object instanceof String) {
                list.add((String) map.get(key));
            } else if (object != null) {
                list.add(object.toString());
            }
        }
    }

    @SuppressWarnings("unchecked")
    public String formatTextualHoldings(Map<String, Object> service) {
        List<String> list = new ArrayList<>();
        if (service.containsKey("location")) {
            List<Map<String, Object>> l = (List<Map<String, Object>>) service.get("location");
            for (Map<String, Object> m : l) {
                if (m.containsKey("textualholdings")) {
                    Object object =  m.get("textualholdings");
                    if (object instanceof Collection<?> collection) {
                        collection.forEach(o -> list.add(o.toString()));
                    } else if (object != null) {
                        list.add(object.toString());
                    }
                }
            }
        }
        return String.join(", ", list.stream()
                .filter(Objects::nonNull)
                .filter(s -> !s.isEmpty())
                .toList());
    }

    public String formatMaterial(Map<String, Object> service) {
        final List<String> result = new ArrayList<>();
        Object object = service.get("material");
        if (object instanceof Collection<?> collection) {
            collection.forEach(o -> result.add(o.toString()));
        } else if (object != null) {
            result.add(object.toString());
        }
        return String.join(", ", result.stream()
                .filter(Objects::nonNull)
                .filter(s -> !s.isEmpty())
                .map(this::findMaterial)
                .toList());
    }

    @SuppressWarnings("unchecked")
    private String formatMode(Map<String, Object> service) {
        List<String> result = new ArrayList<>();
        if (service.containsKey("mode")) {
            Object object = service.get("mode");
            if (object instanceof Collection<?> collection) {
                collection.forEach(o -> result.add(o.toString()));
            } else if (object != null) {
                result.add(object.toString());
            }
        }
        if (service.containsKey("comment")) {
            Object object = service.get("comment");
            if (object instanceof Collection<?> collection) {
                collection.forEach(o -> result.add(o.toString()));
            } else if (object != null) {
                result.add(object.toString());
            }
        }
        return String.join(", ", result.stream()
                .filter(Objects::nonNull)
                .filter(s -> !s.isEmpty())
                .map(this::findMode)
                .toList());
    }

    private String formatPriority(Map<String, Object> service) {
        if (service.containsKey("priority")) {
            return service.get("priority").toString();
        }
        return "";
    }

    private String findMaterial(String value) {
        return materials.stream()
                .filter(m -> m.getValue().equalsIgnoreCase(value))
                .map(AvailForm.Key::getLabel)
                .findFirst().orElse(value);
    }

    private String findMode(String value) {
        return modes.stream()
                .filter(m -> m.getValue().equalsIgnoreCase(value))
                .map(AvailForm.Key::getLabel)
                .findFirst().orElse(value);
    }
}
